@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.75 2000/11/17 01:37:39 astyanax Exp $ */
+/* $Id: nano.c,v 1.76 2000/11/21 06:20:19 astyanax Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -2117,7 +2117,10 @@ int main(int argc, char *argv[])
 #endif
 
     titlebar();
-    if (argc == 1)
+
+    /* Now we check to see if argv[optind] is non-null to determine if
+       we're dealing with a new file or not, not argc == 1... */
+    if (argv[optind] == NULL)
 	new_file();
     else
 	open_file(filename, 0, 0);
