@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.17 2000/09/06 13:39:17 astyanax Exp $ */
+/* $Id: proto.h,v 1.24 2000/11/14 18:25:26 astyanax Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -35,11 +35,14 @@ extern int placewewant;
 extern int mark_beginx, samelinewrap;
 extern int totsize, temp_opt;
 extern int fill, flags,tabsize;
+extern int search_last_line;
 
 extern WINDOW *edit, *topwin, *bottomwin;
 extern char filename[PATH_MAX];
-extern char answer[132];
+extern char *answer;
 extern char *hblank, *help_text;
+extern char *last_search;
+extern char *last_replace;
 extern struct stat fileinfo;
 extern filestruct *current, *fileage, *edittop, *editbot, *filebot; 
 extern filestruct *cutbuffer, *mark_beginbuf;
@@ -47,7 +50,7 @@ extern shortcut *shortcut_list;
 extern shortcut main_list[MAIN_LIST_LEN], whereis_list[WHEREIS_LIST_LEN];
 extern shortcut replace_list[REPLACE_LIST_LEN], goto_list[GOTO_LIST_LEN];
 extern shortcut writefile_list[WRITEFILE_LIST_LEN], help_list[HELP_LIST_LEN];
-extern shortcut spell_list[SPELL_LIST_LEN];
+extern shortcut spell_list[SPELL_LIST_LEN], replace_list_2[REPLACE_LIST_LEN];
 
 #ifdef HAVE_REGEX_H
 extern int use_regexp, regexp_compiled;
@@ -68,7 +71,7 @@ int xplustabs(void);
 int do_yesno(int all, int leavecursor, char *msg, ...);
 int actual_x(filestruct * fileptr, int xplus);
 int strlenpt(char *buf);
-int statusq(shortcut s[], int slen, char *def, char *msg, ...);
+int statusq(int allowtabs, shortcut s[], int slen, char *def, char *msg, ...);
 int write_file(char *name, int tmpfile);
 int do_cut_text(void);
 int do_uncut_text(void);
@@ -77,12 +80,15 @@ int renumber_all(void);
 int open_file(char *filename, int insert, int quiet);
 int do_writeout(int exiting);
 int do_gotoline(long defline);
+int do_replace_loop(char *prevanswer, filestruct *begin, int *beginx,
+			int wholewords, int *i);
 /* Now in move.c */
 int do_up(void);
 int do_down(void);
 int do_left(void);
 int do_right(void);
-
+int check_wildcard_match(const char *text, const char *pattern);
+char *input_tab(char *buf, int place, int *lastWasTab, int *newplace);
 
 void shortcut_init(void);
 void lowercase(char *src);
@@ -107,6 +113,7 @@ void center_cursor(void);
 void bottombars(shortcut s[], int slen);
 void blank_statusbar_refresh(void);
 void *nmalloc (size_t howmuch);
+void *mallocstrcpy(void *dest, void *src);
 void wrap_reset(void);
 void display_main_list(void);
 void nano_small_msg(void);
@@ -119,6 +126,9 @@ void new_magicline(void);
 void splice_node(filestruct *begin, filestruct *new, filestruct *end);
 void null_at(char *data, int index);
 void page_up_center(void);
+void blank_edit(void);
+void search_init_globals(void);
+void replace_abort(void);
 
 int do_writeout_void(void), do_exit(void), do_gotoline_void(void);
 int do_insertfile(void), do_search(void), page_up(void), page_down(void);
@@ -132,3 +142,5 @@ int do_replace(void), do_help(void), do_enter_void(void);
 filestruct *copy_node(filestruct * src);
 filestruct *copy_filestruct(filestruct * src);
 filestruct *make_new_node(filestruct * prevnode);
+filestruct *findnextstr(int quiet, filestruct * begin,
+			int beginx, char *needle);
