@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.45 2000/11/18 03:05:50 astyanax Exp $ */
+/* $Id: search.c,v 1.49 2000/11/21 12:52:55 astyanax Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -129,7 +129,7 @@ int search_init(int replacing)
     /* This is now one simple call.  It just does a lot */
     i = statusq(0, replacing ? replace_list : whereis_list,
 	replacing ? REPLACE_LIST_LEN : WHEREIS_LIST_LEN, backupstring,
-	prompt, reprompt, "");
+	prompt, reprompt, buf);
 
     /* Cancel any search, or just return with no previous search */
     if ((i == -1) || (i < 0 && !last_search[0])) {
@@ -202,21 +202,21 @@ filestruct *findnextstr(int quiet, filestruct * begin, int beginx,
 {
     filestruct *fileptr;
     char *searchstr, *found = NULL, *tmp;
-    int past_editbot = 0;
+    int past_editbot = 0, current_x_find = current_x;
 
     fileptr = current;
 
-    current_x++;
+    current_x_find++;
 
     /* Are we searching the last line? (i.e. the line where search started) */
-    if ((fileptr == begin) && (current_x < beginx))
+    if ((fileptr == begin) && (current_x_find < beginx))
 	search_last_line = 1;
 
     /* Make sure we haven't passed the end of the string */
-    if (strlen(fileptr->data) < current_x)
-	current_x--;
+    if (strlen(fileptr->data) < current_x_find)
+	current_x_find--;
 
-    searchstr = &fileptr->data[current_x];
+    searchstr = &fileptr->data[current_x_find];
 
     /* Look for needle in searchstr */
     while ((found = strstrwrapper(searchstr, needle)) == NULL) {
@@ -490,6 +490,9 @@ int do_replace_loop(char *prevanswer, filestruct *begin, int *beginx,
         }
     }
 
+    if (ISSET(PICO_MODE) && !strcmp(answer, ""))
+	answer = mallocstrcpy(answer, last_replace);
+
     last_replace = mallocstrcpy(last_replace, answer);
     while (1) {
 
