@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.8 2000/09/06 13:39:17 astyanax Exp $ */
+/* $Id: utils.c,v 1.13 2000/11/16 06:01:10 astyanax Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -84,11 +84,11 @@ char *strstrwrapper(char *haystack, char *needle)
 {
 #ifdef HAVE_REGEX_H
     if (ISSET(USE_REGEXP)) {
-      int result=regexec(&search_regexp, haystack, 10, regmatches, 0);
-      if (!result)
-          return haystack+regmatches[0].rm_so;
-      return 0;
-    }  
+	int result = regexec(&search_regexp, haystack, 10, regmatches, 0);
+	if (!result)
+	    return haystack + regmatches[0].rm_so;
+	return 0;
+    }
 #endif
     if (ISSET(CASE_SENSITIVE))
 	return strstr(haystack, needle);
@@ -114,13 +114,36 @@ void *nrealloc(void *ptr, size_t howmuch)
     void *r;
 
     if (!(r = realloc(ptr, howmuch)))
-	die("nano: realloc: out of memory!");
+	die(_("nano: realloc: out of memory!"));
 
     return r;
 }
 
+/* Copy one malloced string to another pointer.
+
+   Should be used as dest = mallocstrcpy(dest, src);
+*/
+void *mallocstrcpy(void *dest, void *src)
+{
+
+    if (dest != NULL)
+	free(dest);
+
+    if (src == NULL) {
+	dest = NULL;
+	return(dest);
+    }
+
+    dest = nmalloc(strlen(src) + 1);
+    strcpy(dest, src);
+
+    return dest;
+}
+
+
 /* Append a new magic-line to filebot */
-void new_magicline(void) {
+void new_magicline(void)
+{
     filebot->next = nmalloc(sizeof(filestruct));
     filebot->next->data = nmalloc(1);
     filebot->next->data[0] = '\0';
@@ -130,3 +153,103 @@ void new_magicline(void) {
     filebot = filebot->next;
     totlines++;
 }
+
+/*
+ * Routine to see if a text string is matched by a wildcard pattern.
+ * Returns TRUE if the text is matched, or FALSE if it is not matched
+ * or if the pattern is invalid.
+ *  *		matches zero or more characters
+ *  ?		matches a single character
+ *  [abc]	matches 'a', 'b' or 'c'
+ *  \c		quotes character c
+ * Adapted from code written by Ingo Wilken, and
+ * then taken from sash, Copyright (c) 1999 by David I. Bell
+ * Permission is granted to use, distribute, or modify this source,
+ * provided that this copyright notice remains intact.
+ * Permission to distribute this code under the GPL has been granted.
+ */
+int check_wildcard_match(const char *text, const char *pattern)
+{
+    const char *retryPat;
+    const char *retryText;
+    int ch;
+    int found;
+    int len;
+
+    retryPat = NULL;
+    retryText = NULL;
+
+    while (*text || *pattern) {
+	ch = *pattern++;
+
+	switch (ch) {
+	case '*':
+	    retryPat = pattern;
+	    retryText = text;
+	    break;
+
+	case '[':
+	    found = FALSE;
+
+	    while ((ch = *pattern++) != ']') {
+		if (ch == '\\')
+		    ch = *pattern++;
+
+		if (ch == '\0')
+		    return FALSE;
+
+		if (*text == ch)
+		    found = TRUE;
+	    }
+	    len = strlen(text);
+	    if (found == FALSE && len != 0) {
+		return FALSE;
+	    }
+	    if (found == TRUE) {
+		if (strlen(pattern) == 0 && len == 1) {
+		    return TRUE;
+		}
+		if (len != 0) {
+		    text++;
+		    continue;
+		}
+	    }
+
+	    /* fall into next case */
+
+	case '?':
+	    if (*text++ == '\0')
+		return FALSE;
+
+	    break;
+
+	case '\\':
+	    ch = *pattern++;
+
+	    if (ch == '\0')
+		return FALSE;
+
+	    /* fall into next case */
+
+	default:
+	    if (*text == ch) {
+		if (*text)
+		    text++;
+		break;
+	    }
+
+	    if (*text) {
+		pattern = retryPat;
+		text = ++retryText;
+		break;
+	    }
+
+	    return FALSE;
+	}
+
+	if (pattern == NULL)
+	    return FALSE;
+    }
+
+    return TRUE;
+}
