@@ -1,4 +1,4 @@
-/* $Id: winio.c,v 1.35 2000/09/15 03:31:09 astyanax Exp $ */
+/* $Id: winio.c,v 1.53 2000/11/17 01:37:39 astyanax Exp $ */
 /**************************************************************************
  *   winio.c                                                              *
  *                                                                        *
@@ -192,7 +192,6 @@ void blank_edit(void)
     int i;
     for (i = 0; i <= editwinrows - 1; i++)
 	mvwaddstr(edit, i, 0, hblank);
-    wrefresh(edit);
 }
 
 
@@ -218,51 +217,89 @@ void check_statblank(void)
     }
 }
 
-/* Get the input from the kb, this should only be called from statusq */
-int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
+/* Repaint the statusbar when getting a character in nanogetstr */
+void nanoget_repaint(char *buf, char *inputbuf, int x)
 {
-    int kbinput = 0, j = 0, x = 0, xend;
-    int x_left = 0;
-    char inputstr[132], inputbuf[132] = "";
+    int len = strlen(buf);
+    int wid = COLS - len;
 
     blank_statusbar();
-    mvwaddstr(bottomwin, 0, 0, buf);
-    if (strlen(def) > 0)
-	waddstr(bottomwin, def);
-    wrefresh(bottomwin);
+    if (x <= COLS - 1) {
+	/* Black magic */
+	buf[len - 1] = ' ';
+
+	mvwaddstr(bottomwin, 0, 0, buf);
+	waddnstr(bottomwin, inputbuf, wid);
+	wmove(bottomwin, 0, (x % COLS));
+    }
+    else {
+	/* Black magic */
+	buf[len - 1] = '$';
+
+	mvwaddstr(bottomwin, 0, 0, buf);
+	waddnstr(bottomwin, &inputbuf[wid * ((x - len) / (wid))], wid);
+	wmove(bottomwin, 0, ((x - len) % wid) + len);
+    }
+}
+
+/* Get the input from the kb, this should only be called from statusq */
+int nanogetstr(int allowtabs, char *buf, char *def, shortcut s[], int slen, 
+	       int start_x)
+{
+    int kbinput = 0, j = 0, x = 0, xend;
+    int x_left = 0, inputlen, tabbed = 0, shift = 0;
+    char *inputbuf;
+    
+    inputbuf = nmalloc(strlen(def) + 1);
+    inputbuf[0] = 0;
 
     x_left = strlen(buf);
     x = strlen(def) + x_left;
 
     /* Get the input! */
-    if (strlen(def) > 0) {
-	strcpy(answer, def);
+    if (strlen(def) > 0)
 	strcpy(inputbuf, def);
-    }
+
+    nanoget_repaint(buf, inputbuf, x);
+
+    /* Make sure any editor screen updates are displayed before getting input */
+    wrefresh(edit);
 
     while ((kbinput = wgetch(bottomwin)) != 13) {
 	for (j = 0; j <= slen - 1; j++) {
 	    if (kbinput == s[j].val) {
-		strcpy(answer, "");
+
+		/* We shouldn't discard the answer it gave, just because
+		   we hit a keystroke, GEEZ! */
+		answer = mallocstrcpy(answer, inputbuf);
+		free(inputbuf);
 		return s[j].val;
 	    }
 	}
 	xend = strlen(buf) + strlen(inputbuf);
 
+	if (kbinput != '\t')
+	    tabbed = 0;
+
 	switch (kbinput) {
+	    /* Stuff we want to equate with <enter>, ASCII 13 */
+	case 343:
+	    ungetch(13);	/* Enter on iris-ansi $TERM, sometimes */
+	    break;
+#ifdef PDCURSES
+	/* Stuff we want to ignore with PDCURSES */
+	case 541:
+	case 542:
+	case 544:
+		break;
+#endif
 	case KEY_HOME:
 	    x = x_left;
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
+	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 	case KEY_END:
 	    x = x_left + strlen(inputbuf);
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
+	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 	case KEY_RIGHT:
 
@@ -277,19 +314,13 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 			strlen(inputbuf) - (x - x_left) - 1);
 		inputbuf[strlen(inputbuf) - 1] = 0;
 	    }
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
+	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 	case NANO_CONTROL_K:
 	case NANO_CONTROL_U:
 	    *inputbuf = 0;
 	    x = x_left;
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
+	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 	case KEY_BACKSPACE:
 	case KEY_DC:
@@ -305,9 +336,19 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 		    inputbuf[strlen(inputbuf) - 1] = 0;
 		}
 	    }
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
+	    if (x > strlen(buf))
+		x--;
+	    nanoget_repaint(buf, inputbuf, x);
+	    break;
+	case NANO_CONTROL_I:
+	    if (allowtabs) {
+		shift = 0;
+		inputbuf = input_tab(inputbuf, (x - x_left), 
+				&tabbed, &shift);
+		x += shift;
+		nanoget_repaint(buf, inputbuf, x);
+	    }
+	    break;
 	case KEY_LEFT:
 	    if (x > strlen(buf))
 		x--;
@@ -323,17 +364,11 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 		switch (kbinput = wgetch(edit)) {
 		case 70:
 		    x = x_left + strlen(inputbuf);
-		    blank_statusbar();
-		    mvwaddstr(bottomwin, 0, 0, buf);
-		    waddstr(bottomwin, inputbuf);
-		    wmove(bottomwin, 0, x);
+		    nanoget_repaint(buf, inputbuf, x);
 		    break;
 		case 72:
 		    x = x_left;
-		    blank_statusbar();
-		    mvwaddstr(bottomwin, 0, 0, buf);
-		    waddstr(bottomwin, inputbuf);
-		    wmove(bottomwin, 0, x);
+		    nanoget_repaint(buf, inputbuf, x);
 		    break;
 		}
 		break;
@@ -351,10 +386,7 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 		    break;
 		case 49:
 		    x = x_left;
-		    blank_statusbar();
-		    mvwaddstr(bottomwin, 0, 0, buf);
-		    waddstr(bottomwin, inputbuf);
-		    wmove(bottomwin, 0, x);
+		    nanoget_repaint(buf, inputbuf, x);
 		    goto skip_126;
 		case 51:
 		    if (strlen(inputbuf) > 0
@@ -364,17 +396,11 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 				strlen(inputbuf) - (x - x_left) - 1);
 			inputbuf[strlen(inputbuf) - 1] = 0;
 		    }
-		    blank_statusbar();
-		    mvwaddstr(bottomwin, 0, 0, buf);
-		    waddstr(bottomwin, inputbuf);
-		    wmove(bottomwin, 0, x);
+		    nanoget_repaint(buf, inputbuf, x);
 		    goto skip_126;
 		case 52:
 		    x = x_left + strlen(inputbuf);
-		    blank_statusbar();
-		    mvwaddstr(bottomwin, 0, 0, buf);
-		    waddstr(bottomwin, inputbuf);
-		    wmove(bottomwin, 0, x);
+		    nanoget_repaint(buf, inputbuf, x);
 		    goto skip_126;
 		  skip_126:
 		    nodelay(edit, TRUE);
@@ -385,27 +411,24 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 		    break;
 		}
 	    }
-	    blank_statusbar();
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
+	    nanoget_repaint(buf, inputbuf, x);
 	    break;
 
-
 	default:
 	    if (kbinput < 32)
 		break;
-	    strcpy(inputstr, inputbuf);
-	    inputstr[x - strlen(buf)] = kbinput;
-	    strcpy(&inputstr[x - strlen(buf) + 1],
-		   &inputbuf[x - strlen(buf)]);
-	    strcpy(inputbuf, inputstr);
+
+	    inputlen = strlen(inputbuf);
+	    inputbuf = nrealloc(inputbuf, inputlen + 2);
+
+	    memmove(&inputbuf[x - x_left + 1], 
+			&inputbuf[x - x_left],
+                        inputlen - (x - x_left) + 1);
+	    inputbuf[x - x_left] = kbinput;
+
 	    x++;
 
-	    mvwaddstr(bottomwin, 0, 0, buf);
-	    waddstr(bottomwin, inputbuf);
-	    wmove(bottomwin, 0, x);
-
+	    nanoget_repaint(buf, inputbuf, x);
 #ifdef DEBUG
 	    fprintf(stderr, _("input \'%c\' (%d)\n"), kbinput, kbinput);
 #endif
@@ -413,9 +436,13 @@ int nanogetstr(char *buf, char *def, shortcut s[], int slen, int start_x)
 	wrefresh(bottomwin);
     }
 
-    strncpy(answer, inputbuf, 132);
+    answer = mallocstrcpy(answer, inputbuf);
+    free(inputbuf);
 
-    if (!strcmp(answer, ""))
+    /* Now that the text is editable instead of bracketed, we have to 
+       check for answer == def, instead of answer == "" */
+    if  (((ISSET(PICO_MODE)) && !strcmp(answer, "")) || 
+	((!ISSET(PICO_MODE)) && !strcmp(answer, def)))
 	return -2;
     else
 	return 0;
@@ -625,110 +652,106 @@ void edit_add(filestruct * fileptr, int yval, int start, int virt_cur_x,
 #ifndef NANO_SMALL
     /* There are quite a few cases that could take place, we'll deal
      * with them each in turn */
-    if (ISSET(MARK_ISSET)    
+    if (ISSET(MARK_ISSET)
 	&& !((fileptr->lineno > mark_beginbuf->lineno
-	     && fileptr->lineno > current->lineno)
-	    || (fileptr->lineno < mark_beginbuf->lineno
-		&& fileptr->lineno < current->lineno)))
-    {
-	    /* If we get here we are on a line that is atleast
-	     * partially selected.  The lineno checks above determined
-	     * that */
-	    if (fileptr != mark_beginbuf && fileptr != current) {
-		/* We are on a completely marked line, paint it all
-		 * inverse */
+	      && fileptr->lineno > current->lineno)
+	     || (fileptr->lineno < mark_beginbuf->lineno
+		 && fileptr->lineno < current->lineno))) {
+	/* If we get here we are on a line that is atleast
+	 * partially selected.  The lineno checks above determined
+	 * that */
+	if (fileptr != mark_beginbuf && fileptr != current) {
+	    /* We are on a completely marked line, paint it all
+	     * inverse */
+	    wattron(edit, A_REVERSE);
+	    mvwaddnstr(edit, yval, 0, fileptr->data, COLS);
+	    wattroff(edit, A_REVERSE);
+	} else if (fileptr == mark_beginbuf && fileptr == current) {
+	    /* Special case, we're still on the same line we started
+	     * marking -- so we call our helper function */
+	    if (virt_cur_x < virt_mark_beginx) {
+		/* To the right of us is marked */
+		add_marked_sameline(virt_cur_x, virt_mark_beginx,
+				    fileptr, yval, virt_cur_x, this_page);
+	    } else {
+		/* To the left of us is marked */
+		add_marked_sameline(virt_mark_beginx, virt_cur_x,
+				    fileptr, yval, virt_cur_x, this_page);
+	    }
+	} else if (fileptr == mark_beginbuf) {
+	    /*
+	     * we're updating the line that was first marked
+	     * but we're not currently on it.  So we want to
+	     * figur out which half to invert based on our
+	     * relative line numbers.
+	     *
+	     * i.e. If we're above the "beginbuf" line, we want to
+	     * mark the left side.  Otherwise we're below, so we
+	     * mark the right
+	     */
+	    int target;
+
+	    if (mark_beginbuf->lineno > current->lineno)
 		wattron(edit, A_REVERSE);
-		mvwaddnstr(edit, yval, 0, fileptr->data, COLS);
+
+	    target =
+		(virt_mark_beginx <
+		 COLS - 1) ? virt_mark_beginx : COLS - 1;
+
+	    mvwaddnstr(edit, yval, 0, fileptr->data, target);
+
+	    if (mark_beginbuf->lineno < current->lineno)
+		wattron(edit, A_REVERSE);
+	    else
 		wattroff(edit, A_REVERSE);
-	    } else if (fileptr == mark_beginbuf && fileptr == current) {
-		/* Special case, we're still on the same line we started
-		 * marking -- so we call our helper function */
-		if (virt_cur_x < virt_mark_beginx) {
-		    /* To the right of us is marked */
-		    add_marked_sameline(virt_cur_x, virt_mark_beginx,
-					fileptr, yval, virt_cur_x,
-					this_page);
-		} else {
-		    /* To the left of us is marked */
-		    add_marked_sameline(virt_mark_beginx, virt_cur_x,
-					fileptr, yval, virt_cur_x,
-					this_page);
-		}
-	    } else if (fileptr == mark_beginbuf) {
-		/*
-		 * we're updating the line that was first marked
-		 * but we're not currently on it.  So we want to
-		 * figur out which half to invert based on our
-		 * relative line numbers.
-		 *
-		 * i.e. If we're above the "beginbuf" line, we want to
-		 * mark the left side.  Otherwise we're below, so we
-		 * mark the right
-		 */
-		int target;
 
-		if (mark_beginbuf->lineno > current->lineno)
-		    wattron(edit, A_REVERSE);
+	    target = (COLS - 1) - virt_mark_beginx;
+	    if (target < 0)
+		target = 0;
 
-		target =
-		    (virt_mark_beginx <
-		     COLS - 1) ? virt_mark_beginx : COLS - 1;
+	    mvwaddnstr(edit, yval, virt_mark_beginx,
+		       &fileptr->data[virt_mark_beginx], target);
 
-		mvwaddnstr(edit, yval, 0, fileptr->data, target);
+	    if (mark_beginbuf->lineno < current->lineno)
+		wattroff(edit, A_REVERSE);
 
-		if (mark_beginbuf->lineno < current->lineno)
-		    wattron(edit, A_REVERSE);
-		else
-		    wattroff(edit, A_REVERSE);
+	} else if (fileptr == current) {
+	    /* we're on the cursors line, but it's not the first
+	     * one we marked.  Similar to the previous logic. */
+	    int this_page_start = get_page_start_virtual(this_page),
+		this_page_end = get_page_end_virtual(this_page);
 
-		target = (COLS - 1) - virt_mark_beginx;
-		if (target < 0)
-		    target = 0;
+	    if (mark_beginbuf->lineno < current->lineno)
+		wattron(edit, A_REVERSE);
 
-		mvwaddnstr(edit, yval, virt_mark_beginx,
-			   &fileptr->data[virt_mark_beginx], target);
+	    if (virt_cur_x > COLS - 2) {
+		mvwaddnstr(edit, yval, 0,
+			   &fileptr->data[this_page_start],
+			   virt_cur_x - this_page_start);
+	    } else {
+		mvwaddnstr(edit, yval, 0, fileptr->data, virt_cur_x);
+	    }
 
-		if (mark_beginbuf->lineno < current->lineno)
-		    wattroff(edit, A_REVERSE);
+	    if (mark_beginbuf->lineno > current->lineno)
+		wattron(edit, A_REVERSE);
+	    else
+		wattroff(edit, A_REVERSE);
 
-	    } else if (fileptr == current) {
-		/* we're on the cursors line, but it's not the first
-		 * one we marked.  Similar to the previous logic. */
-		int this_page_start = get_page_start_virtual(this_page),
-		    this_page_end = get_page_end_virtual(this_page);
+	    if (virt_cur_x > COLS - 2)
+		mvwaddnstr(edit, yval, virt_cur_x - this_page_start,
+			   &fileptr->data[virt_cur_x],
+			   this_page_end - virt_cur_x);
+	    else
+		mvwaddnstr(edit, yval, virt_cur_x,
+			   &fileptr->data[virt_cur_x], COLS - virt_cur_x);
 
-		if (mark_beginbuf->lineno < current->lineno)
-		    wattron(edit, A_REVERSE);
-
-		if (virt_cur_x > COLS - 2) {
-		    mvwaddnstr(edit, yval, 0,
-			       &fileptr->data[this_page_start],
-			       virt_cur_x - this_page_start);
-		} else {
-		    mvwaddnstr(edit, yval, 0, fileptr->data, virt_cur_x);
-		}
-
-		if (mark_beginbuf->lineno > current->lineno)
-		    wattron(edit, A_REVERSE);
-		else
-		    wattroff(edit, A_REVERSE);
-
-		if (virt_cur_x > COLS - 2)
-		    mvwaddnstr(edit, yval, virt_cur_x - this_page_start,
-			       &fileptr->data[virt_cur_x],
-			       this_page_end - virt_cur_x);
-		else
-		    mvwaddnstr(edit, yval, virt_cur_x,
-			       &fileptr->data[virt_cur_x],
-			       COLS - virt_cur_x);
-
-		if (mark_beginbuf->lineno > current->lineno)
-		    wattroff(edit, A_REVERSE);
+	    if (mark_beginbuf->lineno > current->lineno)
+		wattroff(edit, A_REVERSE);
 	}
 
     } else
 #endif
-    /* Just paint the string (no mark on this line) */
+	/* Just paint the string (no mark on this line) */
 	mvwaddnstr(edit, yval, 0, &fileptr->data[start],
 		   get_page_end_virtual(this_page) - start);
 }
@@ -780,9 +803,8 @@ void update_line(filestruct * fileptr, int index)
 		virt_cur_x--;
 	    if (i < mark_beginx)
 		virt_mark_beginx--;
-	} 
-	else if (realdata[i] >= 1 && realdata[i] <= 26) {
-	/* Treat control characters as ^letter */
+	} else if (realdata[i] >= 1 && realdata[i] <= 26) {
+	    /* Treat control characters as ^letter */
 	    fileptr->data[pos++] = '^';
 	    fileptr->data[pos++] = realdata[i] + 64;
 	} else {
@@ -848,12 +870,10 @@ void edit_refresh(void)
     }
     /* If noloop == 1, then we already did an edit_update without finishing
        this function.  So we don't run edit_update again */
-    if (!currentcheck && !noloop) /* Then current has run off the screen... */ 
-    {
+    if (!currentcheck && !noloop) {	/* Then current has run off the screen... */
 	edit_update(current, CENTER);
 	noloop = 1;
-    }
-    else if (noloop)
+    } else if (noloop)
 	noloop = 0;
 
     if (lines <= editwinrows - 1)
@@ -890,8 +910,7 @@ void edit_update(filestruct * fileptr, int topmidbot)
 	return;
 
     temp = fileptr;
-    if (topmidbot == 2)
-	;
+    if (topmidbot == 2);
     else if (topmidbot == 0)
 	for (i = 0; i <= editwinrows - 1 && temp->prev != NULL; i++)
 	    temp = temp->prev;
@@ -933,8 +952,10 @@ void update_cursor(void)
  * global.  Returns -1 on aborted enter, -2 on a blank string, and 0
  * otherwise, the valid shortcut key caught, Def is any editable text we
  * want to put up by default.
+ *
+ * New arg tabs tells whether or not to allow tab completion.
  */
-int statusq(shortcut s[], int slen, char *def, char *msg, ...)
+int statusq(int tabs, shortcut s[], int slen, char *def, char *msg, ...)
 {
     va_list ap;
     char foo[133];
@@ -948,7 +969,7 @@ int statusq(shortcut s[], int slen, char *def, char *msg, ...)
     strncat(foo, ": ", 132);
 
     wattron(bottomwin, A_REVERSE);
-    ret = nanogetstr(foo, def, s, slen, (strlen(foo) + 3));
+    ret = nanogetstr(tabs, foo, def, s, slen, (strlen(foo) + 3));
     wattroff(bottomwin, A_REVERSE);
 
     switch (ret) {
@@ -962,7 +983,7 @@ int statusq(shortcut s[], int slen, char *def, char *msg, ...)
     case NANO_CANCEL_KEY:
 	return -1;
     default:
-	blank_statusbar_refresh();
+	blank_statusbar();
     }
 
 #ifdef DEBUG
@@ -1206,7 +1227,7 @@ int do_help(void)
 	}
 
 	if (i > 1) {
-	    
+
 	}
 
 	i = 0;
@@ -1294,10 +1315,10 @@ void dump_buffer_reverse(filestruct * inptr)
 }
 
 /* Fix editbot based on the assumption that edittop is correct */
-void fix_editbot(void) {
+void fix_editbot(void)
+{
     int i;
     editbot = edittop;
-    for(i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
-	&& (editbot != filebot); i++, editbot = editbot->next);
+    for (i = 0; (i <= editwinrows - 1) && (editbot->next != NULL)
+	 && (editbot != filebot); i++, editbot = editbot->next);
 }
-
